def mask_dialog_dynamics(mdl, container_handle, caller_prop_handle=None):

    # Property Registration
    grounding_prop = mdl.prop(container_handle, "grounding")
    phases_prop = mdl.prop(container_handle, "phases")
    filter_type_prop = mdl.prop(container_handle, "filter_type")

    new_value = None
    if caller_prop_handle:
        new_value = mdl.get_property_disp_value(caller_prop_handle)

    # ------------------------------------------------------------------------------------------------------------------
    #  "phases" property code
    # ------------------------------------------------------------------------------------------------------------------
    if caller_prop_handle == phases_prop:

        if new_value == "3":
            mdl.enable_property(grounding_prop)
        else:
            mdl.set_property_disp_value(grounding_prop, False)
            mdl.disable_property(grounding_prop)

    # ------------------------------------------------------------------------------------------------------------------
    #  "filter" property code
    # ------------------------------------------------------------------------------------------------------------------
    elif caller_prop_handle == filter_type_prop:

        rl1_properties = [mdl.prop(container_handle, pname) for pname in ["rl1_resistance", "rl1_inductance"]]
        rc_properties = [mdl.prop(container_handle, pname) for pname in ["rc_resistance", "rc_capacitance"]]
        rl2_properties = [mdl.prop(container_handle, pname) for pname in ["rl2_resistance", "rl2_inductance"]]

        if new_value == "L":
            [mdl.show_property(prop) for prop in rl1_properties]
            [mdl.hide_property(prop) for prop in rl2_properties + rc_properties]
        elif new_value == "LC":
            [mdl.show_property(prop) for prop in rl1_properties + rc_properties]
            [mdl.hide_property(prop) for prop in rl2_properties]
        elif new_value == "LCL":
            [mdl.show_property(prop) for prop in rl1_properties + rc_properties + rl2_properties]


def circuit_dynamics(mdl, container_handle, caller_prop_handle=None, init=False):
    """

    :param mdl:
    :param container_handle:
    :param caller_prop_handle:
    :param init:
    :return:

    """
    # Property Registration
    grounding_prop = mdl.prop(container_handle, "grounding")
    phases_prop = mdl.prop(container_handle, "phases")
    filter_type_prop = mdl.prop(container_handle, "filter_type")

    new_value = None

    if caller_prop_handle:
        new_value = mdl.get_property_value(caller_prop_handle)

    # ------------------------------------------------------------------------------------------------------------------
    #  "grounding" property code
    # ------------------------------------------------------------------------------------------------------------------
    if caller_prop_handle == grounding_prop:
        comp_handle = mdl.get_parent(container_handle)
        phases = mdl.get_property_value(mdl.prop(container_handle, "phases"))
        # port_dynamics function to support container component
        port_dynamics(mdl, container_handle, grounding_prop)
        # Connections and Internal Items
        ntag_handle = mdl.get_item("gnd_src", parent=comp_handle, item_type="tag")
        if new_value:
            xpos, ypos = get_port_const_attributes("N", phases)["pos"]
            gnd_handle = mdl.create_component("core/Ground",
                                              name="gnd",
                                              parent=comp_handle,
                                              position=(xpos, ypos+48))
            mdl.create_connection(ntag_handle, mdl.term(gnd_handle, "node"))
        else:
            gnd_handle = mdl.get_item("gnd", parent=comp_handle)
            if gnd_handle:
                mdl.delete_item(gnd_handle)
            nport_handle = mdl.get_item("N", parent=comp_handle, item_type="port")
            if nport_handle:
                if not mdl.find_connections(ntag_handle, nport_handle):
                    mdl.create_connection(ntag_handle, nport_handle)

    # ------------------------------------------------------------------------------------------------------------------
    #  "phases" property code
    # ------------------------------------------------------------------------------------------------------------------
    elif caller_prop_handle == phases_prop:

        comp_handle = mdl.get_parent(container_handle)
        filter_type = mdl.get_property_disp_value(mdl.prop(container_handle, "filter_type"))
        # port_dynamics function to support container component
        port_dynamics(mdl, container_handle, phases_prop)
        # Connections and Internal Items
        phase_names = ["B", "C"]
        port_handles = [mdl.get_item(pname, parent=comp_handle, item_type="port") for pname in ["B1", "C1"]]
        ground_handle = mdl.get_item("gnd_tag", parent=comp_handle, item_type="tag")
        xpos_ref = mdl.get_position(mdl.get_item("InvLeg_A", parent=comp_handle))[0]
        inv_leg_handles = [mdl.get_item(cname, parent=comp_handle) for cname in ["InvLeg_B", "InvLeg_C"]]
        mod_handles = [mdl.get_item(tname, parent=comp_handle, item_type="tag")
                       for tname in ["from_modB", "from_modC"]]
        term_mod_handles = [mdl.get_item(tname, parent=comp_handle) for tname in ["term_modB", "term_modC"]]
        imeas_handles = [mdl.get_item(tname, parent=comp_handle, item_type="tag")
                         for tname in ["goto_IB", "goto_IC"]]
        ct_imeas_handles = [mdl.get_item(cname, parent=comp_handle) for cname in ["ct_IB", "ct_IC"]]
        vmeas_handles = [mdl.get_item(tname, parent=comp_handle, item_type="tag")
                         for tname in ["goto_VB", "goto_VC"]]
        ct_vmeas_handles = [mdl.get_item(cname, parent=comp_handle) for cname in ["ct_VB", "ct_VC"]]
        switch_handle = mdl.get_item("S1", parent=comp_handle)

        # Controller
        inv_control_handle = mdl.get_item("InvControl", parent=comp_handle)
        inv_control_pos = mdl.get_position(inv_control_handle)
        vlist_handle = mdl.get_item("Vlist", parent=comp_handle)
        vlist_pos = mdl.get_position(vlist_handle)
        ilist_handle = mdl.get_item("Ilist", parent=comp_handle)
        ilist_pos = mdl.get_position(ilist_handle)
        goto_mod_handles = [mdl.get_item(tname, parent=comp_handle, item_type="tag")
                            for tname in ["mod_vb", "mod_vc"]]
        bus_mod_handle = mdl.get_item("Bus Split mod", parent=comp_handle)
        goto_en_handles = [mdl.get_item(tname, parent=comp_handle, item_type="tag")
                           for tname in ["en_swB", "en_swC"]]
        term_en_handles = [mdl.get_item(tname, parent=comp_handle)
                           for tname in ["term_enB", "term_enC"]]

        if new_value == "3":
            # mod_handles
            for cnt, fhandle in enumerate(term_mod_handles):
                if fhandle:
                    mdl.delete_item(fhandle)
            # imeas_handles
            for cnt, ihandle in enumerate(ct_imeas_handles):
                if ihandle:
                    mdl.delete_item(ihandle)
            # vmeas_handles
            for cnt, vhandle in enumerate(ct_vmeas_handles):
                if vhandle:
                    mdl.delete_item(vhandle)
            # term_en
            for cnt, thandle in enumerate(term_en_handles):
                if thandle:
                    mdl.delete_item(thandle)
            # inv_legs
            for cnt, leg_handle in enumerate(inv_leg_handles):
                if not leg_handle:
                    ypos = mdl.get_position(port_handles[cnt])[1]
                    leg_handle = mdl.create_component("Residential Energy/Generic Leg Inverter (avg)",
                                                      name=f"InvLeg_{phase_names[cnt]}",
                                                      parent=comp_handle,
                                                      position=[xpos_ref, ypos])
                    mdl.create_connection(mdl.term(leg_handle, "N"), ground_handle)
                    mdl.create_connection(mdl.term(leg_handle, "L"), port_handles[cnt])
                    mdl.create_connection(mdl.term(leg_handle, "mod"), mod_handles[cnt])
                    mdl.create_connection(mdl.term(leg_handle, "Imeas"), imeas_handles[cnt])
                    mdl.create_connection(mdl.term(leg_handle, "Vmeas"), vmeas_handles[cnt])
                    mdl.create_connection(mdl.term(leg_handle, "sw"), goto_en_handles[cnt])
                    prop_values = ["rl1_resistance", "rl1_inductance", "rc_resistance", "rc_capacitance",
                                   "rl2_resistance", "rl2_inductance", "ts"]
                    [mdl.set_property_value(mdl.prop(leg_handle, prop), prop) for prop in prop_values]
                    mdl.set_property_value(mdl.prop(leg_handle, "filter_type"), filter_type)
                    mdl.set_property_value(mdl.prop(leg_handle, "sw_r"), "pv_r")
            # Control
            mdl.set_property_value(mdl.prop(inv_control_handle, "phases"), "3")
            isel_handle = mdl.get_item("ilist_sel", parent=comp_handle)
            if isel_handle:
                mdl.delete_item(isel_handle)
                mdl.create_connection(mdl.term(ilist_handle, "out"), mdl.term(inv_control_handle, "Imeas"))
            vsel_handle = mdl.get_item("vlist_sel", parent=comp_handle)
            if vsel_handle:
                mdl.delete_item(vsel_handle)
                mdl.create_connection(mdl.term(vlist_handle, "out"), mdl.term(inv_control_handle, "Vmeas"))
            ct_mod_handle = mdl.get_item("ct_mod", parent=comp_handle)
            if ct_mod_handle:
                mdl.delete_item(ct_mod_handle)
                mdl.set_property_value(mdl.prop(bus_mod_handle, "outputs"), "3")
                mdl.create_connection(mdl.term(bus_mod_handle, "out2"), goto_mod_handles[1])
            # switch connections
            """
            if mdl.get_component_type_name(switch_handle) == "Single Non-Ideal Contactor":
                mdl.delete_item(switch_handle)
                ypos = mdl.get_position(mdl.get_item("B1", parent=comp_handle, item_type="port"))[1]
                xpos = mdl.get_position(mdl.get_item("B1", parent=comp_handle, item_type="port"))[0] - 132
                switch_handle = mdl.create_component("Three Phase Non-Ideal Contactor",
                                                     name="S1",
                                                     parent=comp_handle,
                                                     position=(xpos, ypos)
                                                     )
                mdl.set_property_value(mdl.prop(switch_handle, "ctrl_src"), "Model")
                mdl.set_property_value(mdl.prop(switch_handle, "resistance"), "pv_r")
                en_sw_handle = mdl.get_item("en_sw", parent=comp_handle, item_type="tag")
                mdl.create_connection(en_sw_handle, mdl.term(switch_handle, "ctrl_in"))
                inv_handles = [mdl.get_item(leg_name, parent=comp_handle)
                               for leg_name in ["InvLeg_A", "InvLeg_B", "InvLeg_C"]]
                p_handle = [mdl.get_item(pname, parent=comp_handle, item_type="port")
                            for pname in ["A1", "B1", "C1"]]
                sw_term_handles = [mdl.term(switch_handle, pname) for
                                   pname in ["A1", "B1", "C1", "A2", "B2", "C2"]]
                [mdl.create_connection(mdl.term(lhandle, "L"), to_term) for
                 lhandle, to_term in zip(inv_handles, sw_term_handles[0:3])]
                [mdl.create_connection(from_term, to_term)
                 for from_term, to_term in zip(sw_term_handles[3:], p_handle)]
                 """
        else:
            # inv_legs
            for cnt, leg_handle in enumerate(inv_leg_handles):
                if leg_handle:
                    mdl.delete_item(leg_handle)
            # mod_handles
            for cnt, fhandle in enumerate(mod_handles):
                posx, posy = mdl.get_position(fhandle)
                if not term_mod_handles[cnt]:
                    term_handle = mdl.create_component("core/Termination",
                                                       name=f"term_mod{phase_names[cnt]}",
                                                       parent=comp_handle,
                                                       position=[posx+64, posy])
                    mdl.create_connection(fhandle, mdl.term(term_handle, "in"))
            # imeas_handles
            for cnt, ihandle in enumerate(imeas_handles):
                posx, posy = mdl.get_position(ihandle)
                if not ct_imeas_handles[cnt]:
                    const_handle = mdl.create_component("core/Constant",
                                                        name=f"ct_I{phase_names[cnt]}",
                                                        parent=comp_handle,
                                                        position=[posx-64, posy])
                    mdl.set_property_value(mdl.prop(const_handle, "value"), 0)
                    mdl.set_property_value(mdl.prop(const_handle, "execution_rate"), "ts")
                    mdl.create_connection(mdl.term(const_handle, "out"), ihandle)
            # vmeas_handles
            for cnt, vhandle in enumerate(vmeas_handles):
                posx, posy = mdl.get_position(vhandle)
                if not ct_vmeas_handles[cnt]:
                    const_handle = mdl.create_component("core/Constant",
                                                        name=f"ct_V{phase_names[cnt]}",
                                                        parent=comp_handle,
                                                        position=[posx-64, posy])
                    mdl.set_property_value(mdl.prop(const_handle, "value"), 0)
                    mdl.set_property_value(mdl.prop(const_handle, "execution_rate"), "ts")
                    mdl.create_connection(mdl.term(const_handle, "out"), vhandle)
            # Control
            mdl.set_property_value(mdl.prop(inv_control_handle, "phases"), "1")
            vlist_conn = mdl.find_connections(mdl.term(vlist_handle, "out"), mdl.term(inv_control_handle, "Vmeas"))
            ilist_conn = mdl.find_connections(mdl.term(ilist_handle, "out"), mdl.term(inv_control_handle, "Imeas"))
            if vlist_conn:
                mdl.delete_item(vlist_conn[0])
            if ilist_conn:
                mdl.delete_item(ilist_conn[0])
            vsel_handle = mdl.create_component("core/Bus Selector",
                                               name="vlist_sel",
                                               parent=comp_handle,
                                               position=[vlist_pos[0]+64, vlist_pos[1]])
            mdl.set_property_value(mdl.prop(vsel_handle, "signal_indexes"), "[0]")
            mdl.create_connection(mdl.term(vlist_handle, "out"), mdl.term(vsel_handle, "in"))
            mdl.create_connection(mdl.term(vsel_handle, "out"), mdl.term(inv_control_handle, "Vmeas"))
            isel_handle = mdl.create_component("core/Bus Selector",
                                               name="ilist_sel",
                                               parent=comp_handle,
                                               position=[ilist_pos[0]+64, ilist_pos[1]])
            mdl.set_property_value(mdl.prop(isel_handle, "signal_indexes"), "[0]")
            mdl.create_connection(mdl.term(ilist_handle, "out"), mdl.term(isel_handle, "in"))
            mdl.create_connection(mdl.term(isel_handle, "out"), mdl.term(inv_control_handle, "Imeas"))
            mdl.set_property_value(mdl.prop(bus_mod_handle, "outputs"), "2")
            goto_mod_pos = mdl.get_position(goto_mod_handles[1])
            ct_mod_handle = mdl.create_component("core/Constant",
                                                 name="ct_mod",
                                                 parent=comp_handle,
                                                 position=[goto_mod_pos[0]-128, goto_mod_pos[1]])
            mdl.set_property_value(mdl.prop(ct_mod_handle, "execution_rate"), "ts")
            mdl.create_connection(mdl.term(ct_mod_handle, "out"), goto_mod_handles[1])
            # term_en
            for cnt, thandle in enumerate(goto_en_handles):
                posx, posy = mdl.get_position(thandle)
                if not term_en_handles[cnt]:
                    term_handle = mdl.create_component("core/Termination",
                                                       name=f"term_en{phase_names[cnt]}",
                                                       parent=comp_handle,
                                                       position=[posx+64, posy])
                    mdl.create_connection(thandle, mdl.term(term_handle, "in"))

            # switch connections
            """
            if mdl.get_component_type_name(switch_handle) == "Three Phase Non-Ideal Contactor":
                mdl.delete_item(switch_handle)
                ypos = mdl.get_position(mdl.get_item("A1", parent=comp_handle, item_type="port"))[1]
                xpos = mdl.get_position(mdl.get_item("A1", parent=comp_handle, item_type="port"))[0] - 132
                switch_handle = mdl.create_component("Single Non-Ideal Contactor",
                                                     name="S1",
                                                     parent=comp_handle,
                                                     position=(xpos, ypos)
                                                     )
                mdl.set_property_value(mdl.prop(switch_handle, "ctrl_src"), "Model")
                mdl.set_property_value(mdl.prop(switch_handle, "resistance"), "pv_r")
                en_sw_handle = mdl.get_item("en_sw", parent=comp_handle, item_type="tag")
                mdl.create_connection(en_sw_handle, mdl.term(switch_handle, "ctrl_in"))
                inv_handle = mdl.get_item("InvLeg_A", parent=comp_handle)
                p_handle = mdl.get_item("A1", parent=comp_handle, item_type="port")
                sw_term_handles = [mdl.term(switch_handle, pname) for
                                   pname in ["a_in", "a_out"]]
                mdl.create_connection(mdl.term(inv_handle, "L"), sw_term_handles[0])
                mdl.create_connection(sw_term_handles[1], p_handle)
                """

    # ------------------------------------------------------------------------------------------------------------------
    #  "phases" property code
    # ------------------------------------------------------------------------------------------------------------------
    elif caller_prop_handle == filter_type_prop:

        comp_handle = mdl.get_parent(container_handle)
        inv_handles = [mdl.get_item(inv_name, parent=comp_handle) for inv_name in ["InvLeg_A", "InvLeg_B", "InvLeg_C"]]

        for inv_ph_handle in inv_handles:
            if inv_ph_handle:
                mdl.set_property_value(mdl.prop(inv_ph_handle, "filter_type"), new_value)



def port_dynamics(mdl, container_handle, caller_prop_handle=None, init=False):
    """
    Ports modification (to support container component)

    :param mdl:
    :param container_handle:
    :param caller_prop_handle:
    :param init:
    :return:
    """
    comp_handle = mdl.get_parent(container_handle)
    phases = mdl.get_property_disp_value(mdl.prop(container_handle, "phases"))
    prop_name = None
    new_value = None
    if caller_prop_handle:
        prop_name = mdl.get_name(caller_prop_handle)
        new_value = mdl.get_property_value(caller_prop_handle)

    if prop_name == "grounding":
        if new_value:
            nport_handle = mdl.get_item("N", parent=comp_handle, item_type="port")
            if nport_handle:
                mdl.delete_item(nport_handle)
        else:
            nport_handle = mdl.get_item("N", parent=comp_handle, item_type="port")
            if not nport_handle:
                nport_param = get_port_const_attributes("N", phases)
                nport_handle = mdl.create_port(name=nport_param["name"],
                                               parent=comp_handle,
                                               position=nport_param["pos"],
                                               terminal_position=nport_param["term_pos"],
                                               kind="pe",
                                               flip="flip_horizontal",
                                               label=nport_param["label"])

    elif prop_name == "phases":
        port_names = ["B1", "C1"]
        port_handles = [mdl.get_item(pname, parent=comp_handle, item_type="port") for pname in port_names]
        if new_value == "3":
            for cnt, phandle in enumerate(port_handles):
                if not phandle:
                    port_param = get_port_const_attributes(port_names[cnt], new_value)
                    nport_handle = mdl.create_port(name=port_param["name"],
                                                   parent=comp_handle,
                                                   position=port_param["pos"],
                                                   terminal_position=port_param["term_pos"],
                                                   kind="pe",
                                                   flip="flip_horizontal",
                                                   label=port_param["label"])
        else:
            for cnt, phandle in enumerate(port_handles):
                if phandle:
                    mdl.delete_item(phandle)

        # Update Port Term_positions and labels
        nport_params = get_port_const_attributes("N", new_value)
        nport_handle = mdl.get_item("N", parent=comp_handle, item_type="port")
        if nport_handle:
            mdl.set_port_properties(nport_handle, terminal_position=nport_params["term_pos"],
                                    term_label=nport_params["label"])
        aport_params = get_port_const_attributes("A1", new_value)
        aport_handle = mdl.get_item("A1", parent=comp_handle, item_type="port")
        mdl.set_port_properties(aport_handle, terminal_position=aport_params["term_pos"],
                                term_label=aport_params["label"])


def define_icon(mdl, container_handle):
    """
    Defines the component icon based on its type

    :param mdl: Schematic API
    :param container_handle: Component Handle
    :return: no return
    """
    mdl.set_component_icon_image(container_handle, "./residential_comp_images/res_pv.svg", rotate='no_rotate')
    mdl.set_color(container_handle, "blue")
    #mdl.disp_component_icon_text(container_handle, "avg", size=10, relpos_x=0.8, relpos_y=0.87)


def get_port_const_attributes(port_name, phases):
    """

    """
    if phases == "3":
        a1_label = "A"
        b1_label = "B"
        c1_label = "C"
        n_label = "N"
        n_pos = (64.0, 48.0)
    else:
        a1_label = "L+"
        b1_label = "B"
        c1_label = "C"
        n_label = "L-"
        n_pos = (64.0, -16.0)

    port_dict = {"A1": {"name": "A1", "pos": (7984, 7984), "term_pos": (64.0, -48.0), "label": a1_label},
                 "B1": {"name": "B1", "pos": (7984, 8128), "term_pos": (64.0, -16.0), "label": b1_label},
                 "C1": {"name": "C1", "pos": (7984, 8280), "term_pos": (64.0, 16.0), "label": c1_label},
                 "N": {"name": "N", "pos": (7984, 8408), "term_pos": n_pos, "label": n_label}}

    return port_dict[port_name]


