import math
import numpy as np


def circuit_dynamics(mdl, container_handle, caller_prop_handle=None):

    new_value = None
    prop_name = None
    if caller_prop_handle:
        new_value = mdl.get_property_value(caller_prop_handle)
        prop_name = mdl.get_name(caller_prop_handle)

    # Properties Routines
    # --------------------------------------------------------------------------
    # ZIP Load Type
    # --------------------------------------------------------------------------
    if prop_name in ["load1_type", "load2_type", "load3_type",
                     "load4_type", "load5_type"]:

        comp_handle = mdl.get_parent(container_handle)
        load_name = prop_name.split("_")[0].title()
        load_num = int(load_name[-1])

        # Scada Input / Cte Vars
        enable_name = f"Enable {load_name}"
        enable_handle = mdl.get_item(enable_name, parent=comp_handle)
        enable_pos = None
        if enable_handle:
            enable_pos = mdl.get_position(enable_handle)
        # Bus Join Vars
        bus_join_en_handle = mdl.get_item("BusJoin_en", parent=comp_handle)
        if load_num == 1:
            bus_en_term_name = "in"
        else:
            bus_en_term_name = f"in{load_num-1}"
        if bus_join_en_handle:
            bus_en_term_handle = mdl.term(bus_join_en_handle, bus_en_term_name)
        # LUT Vars
        lut_name = f"LUT_{load_name}"
        lut_handle = mdl.get_item(lut_name, parent=comp_handle)
        lut_pos = None
        if lut_handle:
            lut_pos = mdl.get_position(lut_handle)

        bus_split_lut_handle = mdl.get_item("BusSplit_LUT", parent=comp_handle)
        bus_join_lut_handle = mdl.get_item("BusJoin_LUT", parent=comp_handle)

        if load_num == 1:
            bus_split_lut_term_name = "out"
            bus_join_lut_term_name = "in"
        else:
            bus_split_lut_term_name = f"out{load_num-1}"
            bus_join_lut_term_name = f"in{load_num-1}"

        bus_split_term_handle = mdl.term(bus_split_lut_handle, bus_split_lut_term_name)
        bus_join_term_handle = mdl.term(bus_join_lut_handle, bus_join_lut_term_name)

        if new_value == "Not Used":
            # Enable Logic
            mdl.delete_item(enable_handle)
            enable_handle = mdl.create_component("core/Constant",
                                                 name=enable_name,
                                                 parent=comp_handle,
                                                 position=enable_pos)
            mdl.set_property_value(mdl.prop(enable_handle, "execution_rate"), "ts")
            mdl.set_property_value(mdl.prop(enable_handle, "value"), "0")
            mdl.set_property_value(mdl.prop(enable_handle, "signal_type"), "uint")
            mdl.create_connection(bus_en_term_handle, mdl.term(enable_handle, "out"))
            # LUT logic
            mdl.delete_item(lut_handle)
            lut_handle = mdl.create_component("core/Gain",
                                              name=lut_name,
                                              parent=comp_handle,
                                              position=lut_pos,
                                              hide_name=True)
            mdl.set_property_value(mdl.prop(lut_handle, "gain"), "0")
            mdl.create_connection(bus_split_term_handle, mdl.term(lut_handle, "in"))
            mdl.create_connection(bus_join_term_handle, mdl.term(lut_handle, "out"))
        else:
            # Enable logic
            mdl.delete_item(enable_handle)
            enable_handle = mdl.create_component("core/SCADA Input",
                                                 name=enable_name,
                                                 parent=comp_handle,
                                                 position=enable_pos)
            mdl.set_property_value(mdl.prop(enable_handle, "execution_rate"), "ts")
            mdl.set_property_value(mdl.prop(enable_handle, "max"), "1")
            mdl.set_property_value(mdl.prop(enable_handle, "min"), "0")
            mdl.set_property_value(mdl.prop(enable_handle, "signal_type"), "uint")
            mdl.create_connection(bus_en_term_handle, mdl.term(enable_handle, "out"))
            # LUT logic
            mdl.delete_item(lut_handle)
            lut_handle = mdl.create_component("core/1D look-up table",
                                              name=lut_name,
                                              parent=comp_handle,
                                              position=lut_pos,
                                              hide_name=True)
            mdl.set_property_value(mdl.prop(lut_handle, "in_vec_x"), f"{load_name.lower()}_xarray")
            mdl.set_property_value(mdl.prop(lut_handle, "out_vec_f_x"), f"{load_name.lower()}_yarray")
            mdl.create_connection(bus_split_term_handle, mdl.term(lut_handle, "addr"))
            mdl.create_connection(bus_join_term_handle, mdl.term(lut_handle, "value"))


def mask_dialog_dynamics(mdl, container_handle, caller_prop_handle=None):

    new_value = None
    prop_name = None
    if caller_prop_handle:
        new_value = mdl.get_property_disp_value(caller_prop_handle)
        prop_name = mdl.get_name(caller_prop_handle)

    # Properties Routines
    # --------------------------------------------------------------------------
    # ZIP Load Type
    # --------------------------------------------------------------------------
    if prop_name in ["load1_type", "load2_type", "load3_type",
                     "load4_type", "load5_type"]:

        load_name = prop_name.split("_")[0]
        zip_names = ["plo", "qlo", "zp", "ip", "pp", "zq", "iq", "pq"]
        zip_prop_names = ["_".join([load_name, zip]) for zip in zip_names]
        zip_prop_handles = [mdl.prop(container_handle, zip) for zip in zip_prop_names]
        tst_names = ["transient", "pi", "pf", "tau", "preview"]
        tst_prop_names = ["_".join([load_name, tst]) for tst in tst_names]
        tst_prop_handles = [mdl.prop(container_handle, pname) for pname in tst_prop_names]
        demand_prop_handle = mdl.prop(container_handle, load_name + "_power")
        if new_value == "User Defined":
            [mdl.enable_property(prop) for prop in zip_prop_handles]
            mdl.disable_property(demand_prop_handle)
            mdl.show_property(tst_prop_handles[0])
        else:
            [mdl.disable_property(prop) for prop in zip_prop_handles]
            if new_value == "Not Used":
                mdl.disable_property(demand_prop_handle)
                mdl.set_property_disp_value(tst_prop_handles[0], False)
                [mdl.hide_property(prop) for prop in tst_prop_handles]
            else:
                mdl.enable_property(demand_prop_handle)
                mdl.show_property(tst_prop_handles[0])
            coefs = get_zip_constants(mdl, container_handle, load_name)
            [mdl.set_property_disp_value(prop, round(value, 3))
             for prop, value in zip(zip_prop_handles, coefs)]

    # Properties Routines
    # --------------------------------------------------------------------------
    # Load Power
    # --------------------------------------------------------------------------
    elif prop_name in ["load1_power", "load2_power", "load3_power",
                       "load4_power", "load5_power"]:

        load_name = prop_name.split("_")[0]
        zip_names = ["plo", "qlo"]
        zip_prop_names = ["_".join([load_name, zip]) for zip in zip_names]
        zip_prop_handles = [mdl.prop(container_handle, zip) for zip in zip_prop_names]
        coefs = get_zip_constants(mdl, container_handle, load_name)[0:2]
        [mdl.set_property_disp_value(prop, round(value, 3))
         for prop, value in zip(zip_prop_handles, coefs)]

    # --------------------------------------------------------------------------
    # Transient
    # --------------------------------------------------------------------------
    elif prop_name in ["load1_transient", "load2_transient", "load3_transient",
                       "load4_transient", "load5_transient"]:

        load_name = prop_name.split("_")[0]
        tst_names = ["pi", "pf", "tau", "preview"]
        tst_prop_names = ["_".join([load_name, tst]) for tst in tst_names]
        tst_prop_handles = [mdl.prop(container_handle, tst) for tst in tst_prop_names]

        if new_value == True or new_value == "True":
            [mdl.show_property(prop) for prop in tst_prop_handles]
        else:
            [mdl.hide_property(prop) for prop in tst_prop_handles]


# Using this same function from the first version
def get_zip_constants(mdl, container_handle, load_name):
    prop_type_handle = mdl.prop(container_handle, load_name + "_type")
    prop_power_handle = mdl.prop(container_handle, load_name + "_power")

    load_type = None
    appliance = mdl.get_property_disp_value(prop_type_handle)
    try:
        power = float(mdl.get_property_disp_value(prop_power_handle)) * 1000
    except:
        power = 0
    # Classification
    if appliance == "Incandescent Lamp":
        load_type = "GIL"
    elif appliance == "Fluorescent Lamp":
        load_type = "CFL/LFL"
    elif appliance == "LED Light Sources":
        load_type = "LED LS"
    elif appliance == "Refrigerator/Freezer":
        load_type = "RSIR_CT"
    elif appliance == "Washer dryer (heating)":
        load_type = "Resistive"
    elif appliance == "Washer dryer (drum rotation)":
        load_type = "CSCR_CT"
    elif appliance == "Washer dryer (centrifugal pump)":
        load_type = "RSIR_QT"
    elif appliance == "Washer dryer (water pump)":
        load_type = "CSCR_QT"
    elif appliance == "Electric Oven/Hob/Toaster/Kettle":
        load_type = "Resistive"
    elif appliance == "Microwave Oven":
        load_type = "SMPS_pPFC"
    elif appliance == "Food Processors":
        load_type = "CSCR_CT"
    elif appliance == "Shower/Water/Space heater":
        load_type = "Resistive"
    elif appliance == "Consumer Electronics (Low-Power)":
        load_type = "SMPS_noPFC"
    elif appliance == "Consumer Electronics (LCD/LED TV)":
        load_type = "SMPS_aPFC"
    elif appliance == "Consumer Electronics (CRT/Plasma TV)":
        load_type = "SMPS_pPFC"
    else:
        load_type = "Not Used"

    # Constants
    if load_type == "Not Used":
        pf = 0
        sign = 0
        z_p, i_p, p_p = [0.00, 0.00, 0.00]
        z_q, i_q, p_q = [0.00, 0.00, 0.00]
    elif load_type == "GIL":
        pf = 1
        sign = 1
        z_p, i_p, p_p = [0.43, 0.69, -0.12]
        z_q, i_q, p_q = [0.00, 0.00, 0.00]
    elif load_type == "CFL/LFL":
        pf = 0.91
        sign = -1
        z_p, i_p, p_p = [-0.01, 0.96, 0.05]
        z_q, i_q, p_q = [0.10, -0.73, -0.37]
    elif load_type == "LED LS":
        pf = 0.137
        sign = -1
        z_p, i_p, p_p = [0.23, 0.85, -0.08]
        z_q, i_q, p_q = [-1.05, 0.04, 0.01]
    elif load_type == "Resistive":
        pf = 1.0
        sign = 1
        z_p, i_p, p_p = [1.00, 0.00, 0.00]
        z_q, i_q, p_q = [0.00, 0.00, 0.00]
    elif load_type == "SMPS_noPFC":
        pf = 0.994
        sign = 1
        z_p, i_p, p_p = [0.00, 0.00, 1.00]
        z_q, i_q, p_q = [-3.63, 9.88, -7.25]
    elif load_type == "SMPS_pPFC":
        pf = 0.97
        sign = 1
        z_p, i_p, p_p = [0.00, 0.00, 1.00]
        z_q, i_q, p_q = [0.45, -1.44, 1.99]
    elif load_type == "SMPS_aPFC":
        pf = 1.0
        sign = 1
        z_p, i_p, p_p = [1.00, 0.00, 0.00]
        z_q, i_q, p_q = [0.00, 0.00, 0.00]
    elif load_type == "RSIR_QT":
        pf = 0.62
        sign = 1
        z_p, i_p, p_p = [0.10, 0.10, 0.80]
        z_q, i_q, p_q = [1.40, -0.91, 0.50]
    elif load_type == "RSIR_CT":
        pf = 0.62
        sign = 1
        z_p, i_p, p_p = [0.63, -1.20, 1.57]
        z_q, i_q, p_q = [1.40, -0.91, 0.50]
    elif load_type == "CSCR_CT":
        pf = 0.90
        sign = 1
        z_p, i_p, p_p = [0.50, -0.62, 1.11]
        z_q, i_q, p_q = [1.54, -1.43, 0.89]
    elif load_type == "CSCR_QT":
        pf = 0.90
        sign = 1
        z_p, i_p, p_p = [0.22, 0.08, 0.69]
        z_q, i_q, p_q = [1.54, -1.43, 0.89]
    plo = power * pf
    qlo = sign * np.sqrt(power * power - plo * plo)

    return [plo / 1000, qlo / 1000, z_p, i_p, p_p, z_q, i_q, p_q]


def get_lut_values(mdl, container_handle, load_name):
    comp_handle = mdl.get_parent(container_handle)
    enable = mdl.get_property_disp_value(mdl.prop(container_handle, load_name + "_transient"))

    if enable:

        try:
            pf = float(mdl.get_property_disp_value(mdl.prop(container_handle, load_name + "_pf")))
        except:
            try:
                pf = mdl.get_ns_var(mdl.get_property_disp_value(mdl.prop(container_handle, load_name + "_pf")))
            except:
                msg = f"It wasn't possible to evaluate the property {load_name}_pf."
                mdl.warning(msg, kind="Property Evaluation", context=comp_handle)
            return [], []

        try:
            pi = float(mdl.get_property_disp_value(mdl.prop(container_handle, load_name + "_pi")))
        except:
            try:
                pi = mdl.get_ns_var(mdl.get_property_disp_value(mdl.prop(container_handle, load_name + "_pi")))
            except:
                msg = f"It wasn't possible to evaluate the property {load_name}_pi."
                mdl.warning(msg, kind="Property Evaluation", context=comp_handle)
                return [], []

        try:
            tau = float(mdl.get_property_disp_value(mdl.prop(container_handle, load_name + "_tau")))
        except:
            try:
                tau = mdl.get_ns_var(mdl.get_property_disp_value(mdl.prop(container_handle, load_name + "_tau")))
            except:
                msg = f"It wasn't possible to evaluate the property {load_name}_tau."
                mdl.warning(msg, kind="Property Evaluation", context=comp_handle)
                return [], []

        x = np.arange(0, tau * 6, 1e-2)
        x = x.tolist()
        y = [0] * len(x)
        for cnt, value in enumerate(x):
            y[cnt] = pf + (pi - pf) * math.exp(-value / tau)
    else:
        x = [0, 1]
        y = [1, 1]

    return x, y


def pre_compile(mdl, container_handle):

    # Draft for future purposes

    dict_vars = dict()

    # Array of LUTs
    dict_vars["load1_xarray"], dict_vars["load1_yarray"] = get_lut_values(mdl, container_handle, "load1")
    dict_vars["load2_xarray"], dict_vars["load2_yarray"] = get_lut_values(mdl, container_handle, "load2")
    dict_vars["load3_xarray"], dict_vars["load3_yarray"] = get_lut_values(mdl, container_handle, "load3")
    dict_vars["load4_xarray"], dict_vars["load4_yarray"] = get_lut_values(mdl, container_handle, "load4")
    dict_vars["load5_xarray"], dict_vars["load5_yarray"] = get_lut_values(mdl, container_handle, "load5")

    return dict_vars


def define_icon(mdl, container_handle):
    """
    Defines the component icon based on its type

    :param mdl: Schematic API
    :param container_handle: Component Handle
    :return: no return
    """
    mdl.set_component_icon_image(container_handle, "./residential_comp_images/res_load.svg", rotate="no_rotate")
    mdl.set_color(container_handle, "blue")
    mdl.disp_component_icon_text(container_handle, "ZIP Based", size=10, relpos_x=0.5, relpos_y=0.93, rotate="no_rotate")
